# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pagetemplates.ui'
#

#      by: PyQt4 UI code generator 4.5.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(605, 375)
        self.verticalLayout_4 = QtGui.QVBoxLayout(Form)
        self.verticalLayout_4.setMargin(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.splitter = QtGui.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtGui.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.formLayout = QtGui.QFormLayout(self.layoutWidget)
        self.formLayout.setObjectName("formLayout")
        self.label_2 = QtGui.QLabel(self.layoutWidget)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtGui.QFormLayout.LabelRole, self.label_2)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.templates = QtGui.QComboBox(self.layoutWidget)
        self.templates.setObjectName("templates")
        self.horizontalLayout_2.addWidget(self.templates)
        self.newTemplate = QtGui.QToolButton(self.layoutWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/new.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.newTemplate.setIcon(icon)
        self.newTemplate.setObjectName("newTemplate")
        self.horizontalLayout_2.addWidget(self.newTemplate)
        self.delTemplate = QtGui.QToolButton(self.layoutWidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/close.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.delTemplate.setIcon(icon1)
        self.delTemplate.setObjectName("delTemplate")
        self.horizontalLayout_2.addWidget(self.delTemplate)
        self.formLayout.setLayout(0, QtGui.QFormLayout.FieldRole, self.horizontalLayout_2)
        self.label = QtGui.QLabel(self.layoutWidget)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtGui.QFormLayout.LabelRole, self.label)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.frames = QtGui.QComboBox(self.layoutWidget)
        self.frames.setObjectName("frames")
        self.horizontalLayout.addWidget(self.frames)
        self.newFrame = QtGui.QToolButton(self.layoutWidget)
        self.newFrame.setIcon(icon)
        self.newFrame.setObjectName("newFrame")
        self.horizontalLayout.addWidget(self.newFrame)
        self.delFrame = QtGui.QToolButton(self.layoutWidget)
        self.delFrame.setIcon(icon1)
        self.delFrame.setObjectName("delFrame")
        self.horizontalLayout.addWidget(self.delFrame)
        self.formLayout.setLayout(1, QtGui.QFormLayout.FieldRole, self.horizontalLayout)
        self.label_3 = QtGui.QLabel(self.layoutWidget)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtGui.QFormLayout.LabelRole, self.label_3)
        self.top = QtGui.QLineEdit(self.layoutWidget)
        self.top.setObjectName("top")
        self.formLayout.setWidget(2, QtGui.QFormLayout.FieldRole, self.top)
        self.label_4 = QtGui.QLabel(self.layoutWidget)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(3, QtGui.QFormLayout.LabelRole, self.label_4)
        self.left = QtGui.QLineEdit(self.layoutWidget)
        self.left.setObjectName("left")
        self.formLayout.setWidget(3, QtGui.QFormLayout.FieldRole, self.left)
        self.label_5 = QtGui.QLabel(self.layoutWidget)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(4, QtGui.QFormLayout.LabelRole, self.label_5)
        self.width = QtGui.QLineEdit(self.layoutWidget)
        self.width.setObjectName("width")
        self.formLayout.setWidget(4, QtGui.QFormLayout.FieldRole, self.width)
        self.label_6 = QtGui.QLabel(self.layoutWidget)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(5, QtGui.QFormLayout.LabelRole, self.label_6)
        self.height = QtGui.QLineEdit(self.layoutWidget)
        self.height.setObjectName("height")
        self.formLayout.setWidget(5, QtGui.QFormLayout.FieldRole, self.height)
        self.header = QtGui.QCheckBox(self.layoutWidget)
        self.header.setObjectName("header")
        self.formLayout.setWidget(6, QtGui.QFormLayout.FieldRole, self.header)
        self.footer = QtGui.QCheckBox(self.layoutWidget)
        self.footer.setObjectName("footer")
        self.formLayout.setWidget(7, QtGui.QFormLayout.FieldRole, self.footer)
        self.label_7 = QtGui.QLabel(self.layoutWidget)
        self.label_7.setObjectName("label_7")
        self.formLayout.setWidget(8, QtGui.QFormLayout.LabelRole, self.label_7)
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.background = QtGui.QLineEdit(self.layoutWidget)
        self.background.setObjectName("background")
        self.horizontalLayout_3.addWidget(self.background)
        self.selectFile = QtGui.QToolButton(self.layoutWidget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/file.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.selectFile.setIcon(icon2)
        self.selectFile.setObjectName("selectFile")
        self.horizontalLayout_3.addWidget(self.selectFile)
        self.formLayout.setLayout(8, QtGui.QFormLayout.FieldRole, self.horizontalLayout_3)
        self.verticalLayout_4.addWidget(self.splitter)
        self.label_2.setBuddy(self.templates)
        self.label_3.setBuddy(self.top)
        self.label_4.setBuddy(self.left)
        self.label_5.setBuddy(self.width)
        self.label_6.setBuddy(self.height)
        self.label_7.setBuddy(self.background)

        self.retranslateUi(Form)
        QtCore.QObject.connect(self.top, QtCore.SIGNAL("textEdited(QString)"), Form.applyChanges)
        QtCore.QObject.connect(self.left, QtCore.SIGNAL("textEdited(QString)"), Form.applyChanges)
        QtCore.QObject.connect(self.width, QtCore.SIGNAL("textEdited(QString)"), Form.applyChanges)
        QtCore.QObject.connect(self.height, QtCore.SIGNAL("textEdited(QString)"), Form.applyChanges)
        QtCore.QObject.connect(self.header, QtCore.SIGNAL("toggled(bool)"), Form.applyChanges)
        QtCore.QObject.connect(self.footer, QtCore.SIGNAL("toggled(bool)"), Form.applyChanges)
        QtCore.QObject.connect(self.background, QtCore.SIGNAL("textEdited(QString)"), Form.applyChanges)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("Form", "Page Template:", None, QtGui.QApplication.UnicodeUTF8))
        self.newTemplate.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.delTemplate.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Form", "Frame:", None, QtGui.QApplication.UnicodeUTF8))
        self.newFrame.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.delFrame.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("Form", "Top:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("Form", "Left:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_5.setText(QtGui.QApplication.translate("Form", "Width:", None, QtGui.QApplication.UnicodeUTF8))
        self.label_6.setText(QtGui.QApplication.translate("Form", "Height:", None, QtGui.QApplication.UnicodeUTF8))
        self.header.setText(QtGui.QApplication.translate("Form", "Show Header", None, QtGui.QApplication.UnicodeUTF8))
        self.footer.setText(QtGui.QApplication.translate("Form", "Show Footer", None, QtGui.QApplication.UnicodeUTF8))
        self.label_7.setText(QtGui.QApplication.translate("Form", "Background:", None, QtGui.QApplication.UnicodeUTF8))
        self.selectFile.setText(QtGui.QApplication.translate("Form", "...", None, QtGui.QApplication.UnicodeUTF8))

import icons_rc

if __name__ == "__main__":
    import sys
    app = QtGui.QApplication(sys.argv)
    Form = QtGui.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())

